/****************************<--OGI-->*************************************/
/*                                                                        */
/*             Center for Spoken Language Understanding                   */
/*        Oregon Graduate Institute of Science & Technology               */
/*                         Portland, OR USA                               */
/*                        Copyright (c) 1999                              */
/*                                                                        */
/*      This module is not part of the CSTR/University of Edinburgh       */
/*               release of the Festival TTS system.                      */
/*                                                                        */
/*  In addition to any conditions disclaimers below, please see the file  */
/*  "license_ogi_tts.txt" distributed with this software for information  */
/*  on usage and redistribution, and for a DISCLAIMER OF ALL WARRANTIES.  */
/*                                                                        */
/****************************<--OGI-->*************************************/
#include "OGI_WaveChunk.h"

#define CHUNK_OUT_OF_RANGE -1

  // which chunk is sample 'samp' in?
int OGI_WaveChunk::which_chunk(int samp){
  int out=-1;
  EST_Litem *s;
  for (s=true_samp.head(); s!=0; s=next(s)){
    if (true_samp(s) > samp)
      break;
    out++;
  }
  
  if ((out==-1) || (out >= Nchunks())){
    return CHUNK_OUT_OF_RANGE;
    EST_error("OGI_WaveChunk couldn't locate sample %d in OGI_WaveChunk (size=%d)\n",samp,length());
  }
  return out;
}

void  OGI_WaveChunk::append_chunk(EST_Wave &wav, int B, int E, int combineflag){

  int Nzeros=0;
  int joinpt=0;

  if (B < 0){
    Nzeros = -B;
    joinpt = true_samp.last();
    B = 0;
  }
  
  // if combine flag set, just merge wav with last chunk
  //   and discard samples after end.last() and before B
  if (combineflag){
    chunks.resize(end.last(), EST_ALL, 1);
    EST_Wave sub;
    wav.sub_wave(sub, B, EST_ALL); 
    chunks += sub;
    
    end.last() = end.last() + E-B;
    true_samp.last() = true_samp.last() + E-B;
    offset.last() = chunks.length();
  }
  else{
    start.append(B + offset.last());
    end.append(E + offset.last());
    true_samp.append(true_samp.last() + (end.last()-start.last()));
    
    // append wave and record offset
    set_sample_rate(wav.sample_rate());
    chunks += wav;
    offset.append(chunks.length());
  }
  
  if (Nzeros > 0)
    insert_zeros(joinpt, Nzeros);
}


// adjust end of last segment (for concatenation)
void OGI_WaveChunk::adjust_last_segE(int adj){
  if (adj != 0){
    if (end.length() > 0){
      // insert zeros if we need more samples
      if ((end.last() + adj) > chunks.length()){
	chunks.resize(end.last() + adj, EST_ALL, 1);
	offset.last() = end.last() + adj;
      }
      end.last() = end.last() + adj;
      true_samp.last() = true_samp.last() + adj;
    }
    else
      cerr << "OGI_WaveChunk: can't adjust segE before appending something !\n";
  }
}

void OGI_WaveChunk::insert_zeros(int B, int Nzeros){
  int k;
  
  // check B
  if ((B < 0) || (B > true_samp.last()))
    EST_error("OGI_WaveChunk::insert_zeros(): request to insert zeros at point %d beyond end of wave (%d).\n",B,true_samp.last());
  
  // find chunk that contains sample B
  int ch = which_chunk(B);
  if (ch == CHUNK_OUT_OF_RANGE){
    EST_error("OGI_WaveChunk: tried to insert zeros at %d (size=%d)\n",B,length());
  }
  
  // where to insert zeros in chunks
  int Bch = B - true_samp.nth(ch) + offset.nth(ch);
  
  // update {offset, start, end, true_samp} after this point
  end.nth(ch) += Nzeros;
  for (k=ch+1; k<start.length(); k++){
    offset.nth(k)    += Nzeros;
    true_samp.nth(k) += Nzeros;
    start.nth(k)     += Nzeros;
    end.nth(k)     += Nzeros;
  }
  offset.nth(k)    += Nzeros;  // one more offset and true_samp
  true_samp.nth(k) += Nzeros;  //   than start,end 
  
  int Norig = chunks.length();
  chunks.resize(Norig + Nzeros, EST_ALL, 1);
  
  // move end part, set zeros 
  //  (this is probably a dangerous way to do it, since depends on
  //   internals of EST_Wave)
  short *s = chunks.values().memory();
  memmove(&(s[Bch+Nzeros]), &(s[Bch]), (Norig-Bch)*sizeof(short));
  memset(&(s[Bch]), 0, Nzeros*sizeof(short));
}
  

void OGI_WaveChunk::get(int center, int back, int forward, OGI_PitchSyncBuffer &outbuf){
  
  int chnk = which_chunk(center);
  if (chnk == CHUNK_OUT_OF_RANGE){
    outbuf.clear();  // just give back empty buffer
    return;
  }
  
  int next_offset;
  
  if (chnk == offset.length())
    next_offset = chunks.length();
  else
    next_offset = offset.nth(chnk+1);

  // available # samples to back and forward of center
  int avail_back = center - true_samp.nth(chnk) + start.nth(chnk) - offset.nth(chnk);
  int avail_fwd  = true_samp.nth(chnk) + end.nth(chnk)-start.nth(chnk) - center + next_offset - end.nth(chnk);

  // begin, mid, end relative to begin of chunks array
  int Mid = offset.nth(chnk) + avail_back;
  int Begin = Mid - min(back, avail_back);
  int End   = Mid + min(forward, avail_fwd);
  
  outbuf.clear();
  
  // insert zeros to make sure you get the length asked for
  outbuf.set_mid_end(back, back+forward);

  double *out = &(outbuf.buf[back - (Mid-Begin)]);  
  for (int k=Begin; k<End; k++){
    *out++ = (double) chunks.a_no_check(k,0);
  }
}

void OGI_WaveChunk::load_chunk(const EST_String &filename){
  chunks.load(filename);
  start.append(offset.last());
  end.append(chunks.length() + offset.last());
  true_samp.append(true_samp.last() + (end.last()-start.last()));
  offset.append(chunks.length());    
}



//
// dump function that dumps chunks and a label file with offset, start, end, true_samp
//
void OGI_WaveChunk::dump_chunks(const EST_String &basename){
  
  int k,l;
  float Fs = chunks.sample_rate();
  char cc[100];
  EST_StrList names;
  EST_FVector nums(3*start.length());
  EST_String fname;
  
  names.clear();
    
  // create labels
  for (k=0,l=0; k<start.length(); k++){      
    sprintf(cc, "offset");
    names.append(EST_String(cc));
    nums[l++] = start.nth(k)/Fs;
    
    sprintf(cc, "true_samp=%.1f",true_samp.nth(k)/Fs*1000.);
    names.append(EST_String(cc));
    nums[l++] = end.nth(k)/Fs;
    
    sprintf(cc, "end");
    names.append(EST_String(cc));
    nums[l++] = offset.nth(k+1)/Fs;
  }
  
  // wave
  fname = basename + ".wav";
  save_chunks(fname);
  
  // labels
  fname = basename + ".phn";
  save_OGI_label(nums, names, fname);
}


// dump function that dumps "trimmed" chunks and a label file
//   -- just does raw concatenation with no overlap
void OGI_WaveChunk::dump_trimmed(const EST_String &basename){
  
  int k,l;
  float Fs = chunks.sample_rate();
  char cc[100];
  EST_StrList names;
  EST_FVector nums(start.length());
  EST_String fname;
  EST_Wave wav,totwav;
  
  names.clear();
  
  // for each chunk
  for (k=0,l=0; k<start.length(); k++){      
    // create labels
    sprintf(cc, "chunk%d",k);
    names.append(EST_String(cc));
    nums[l++] = (true_samp.nth(k) + end.nth(k)-start.nth(k))/Fs;
    
    // dump active chunks
    chunks.sub_wave(wav, start.nth(k), end.nth(k)-start.nth(k));
    totwav += wav;
  }

  // open out wave
  fname = basename + ".wav";
  totwav.save(fname);
  
  // labels
  fname = basename + ".phn";
  save_OGI_label(nums, names, fname);
}

